<?php

namespace Drupal\ds\Form;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Drupal\ds\Plugin\DsPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure block fields.
 */
class BlockFieldForm extends FieldFormBase implements ContainerInjectionInterface {

  /**
   * The block manager.
   *
   * Manages discovery and instantiation of block plugins.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  private BlockManagerInterface $blockManager;

  /**
   * The BlockFieldForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache_invalidator
   *   The cache invalidator.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\ds\Plugin\DsPluginManager $plugin_manager
   *   The plugin manager service.
   * @param \Drupal\Core\Block\BlockManagerInterface $plugin_manager_block
   *   The block manager service.
   */
  public function __construct(
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    CacheTagsInvalidatorInterface $cache_invalidator,
    ModuleHandlerInterface $module_handler,
    DsPluginManager $plugin_manager,
    BlockManagerInterface $plugin_manager_block
  ) {
    parent::__construct($config_factory, $entity_type_manager, $cache_invalidator, $module_handler, $plugin_manager);
    $this->blockManager = $plugin_manager_block;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('cache_tags.invalidator'),
      $container->get('module_handler'),
      $container->get('plugin.manager.ds'),
      $container->get('plugin.manager.block')
    );
  }

  /**
   * The type of the dynamic ds field.
   */
  const TYPE = 'block';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ds_field_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $field_key = '') {
    $form = parent::buildForm($form, $form_state, $field_key);

    $field = $this->field;

    $manager = $this->blockManager;

    $blocks = [];
    foreach ($manager->getDefinitions() as $plugin_id => $plugin_definition) {
      $extra = '';
      if (strpos($plugin_id, 'field_block:') !== FALSE || strpos($plugin_id, 'inline_block:') !== FALSE) {
        if (Settings::get('ds_exclude_layout_builder_blocks', TRUE)) {
          continue;
        }
        $extra = ' (Layout Builder)';
      }
      $blocks[$plugin_id] = $plugin_definition['admin_label'] . $extra;
    }
    asort($blocks);

    $form['block_identity']['block'] = [
      '#type' => 'select',
      '#options' => $blocks,
      '#title' => $this->t('Block'),
      '#required' => TRUE,
      '#default_value' => $field['properties']['block'] ?? '',
    ];

    $form['use_block_title'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use block title as the field label'),
      '#default_value' => $field['properties']['use_block_title'] ?? FALSE,
      '#weight' => 90,
    ];

    $form['add_block_wrappers'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add block wrappers and classes'),
      '#default_value' => $field['properties']['add_block_wrappers'] ?? FALSE,
      '#description' => $this->t('Render using the block theme hook to add the block wrappers and classes.'),
      '#weight' => 91,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getProperties(FormStateInterface $form_state) {
    $properties['block'] = $form_state->getValue('block');

    // Preserve existing block config.
    $field_key = $form_state->getValue('id');
    $field = $this->config('ds.field.' . $field_key)->get();
    if (isset($field['properties']) && ($field['properties']['block'] == $properties['block'])) {
      $properties = $field['properties'];
    }

    // Save configuration.
    $properties['use_block_title'] = $form_state->getValue('use_block_title');
    $properties['add_block_wrappers'] = $form_state->getValue('add_block_wrappers');

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return BlockFieldForm::TYPE;
  }

  /**
   * {@inheritdoc}
   */
  public function getTypeLabel() {
    return 'Block field';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // Create an instance of the block to find out if it has a config form.
    // Redirect to the block config form if there is one.
    /** @var \Drupal\Core\Block\BlockPluginInterface $block */
    $manager = $this->blockManager;
    $block_id = $this->field['properties']['block'];
    $block = $manager->createInstance($block_id);

    // Create fake form state and inject default theme in form state (Site
    // branding needs it for instance).
    $fake_form_state = new FormState();
    $default_theme = $this->config('system.theme')->get('default');
    $fake_form_state->set('block_theme', $default_theme);
    $block_config_form = $block->blockForm([], $fake_form_state);
    if ($block_config_form) {
      $url = Url::fromRoute('ds.manage_block_field_config', ['field_key' => $this->field['id']]);
      $form_state->setRedirectUrl($url);
    }

    // Invalidate all blocks.
    Cache::invalidateTags(['config:ds.block_base']);
  }

}
